# -*- coding: utf-8 -*-

import os
import time
import logging
import logging.handlers
import json

import click
from kombu import Connection

from shopping_utils.google_shopping import GoogleShopping
from em_tasks.utils.product_service import ProductService
from em_tasks.store.models import *

from em_celery import logger, init_db, get_product_service
from em_celery.tasks.shopify_delete_product_task import delete_shopify_product


@click.command('Send delete shopify blacklisted product task to worker.')
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option('-q', '--qps', type=float, help='Quantity per second (QPS) to send task.')
@click.option('-s', '--store_code', type=str, required=True, help='Store code that products shall be uploaded to.')
def send_shopify_blacklisted_product_delete_task(store_code, broker_url, qps):
  log_dir = os.path.join(os.path.expanduser('~'), '.em_celery', 'logs')
  if not os.path.isdir(log_dir):
    os.makedirs(log_dir)
  log_path = os.path.join(log_dir, 'shopify_products_blacklist_remover.log')
  level = logging.INFO
  max_bytes = 20 * 1024 ** 2
  fh = logging.handlers.RotatingFileHandler(
    log_path, maxBytes=max_bytes, backupCount=5)
  fh.setLevel(level)
  formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
  fh.setFormatter(formatter)
  logger.addHandler(fh)

  init_db()

  queue = 'ShopifyProductDelete'
  product_service = get_product_service()
  remover = ShopifyProductsBlacklistRemover(store_code, product_service, broker_url, queue, qps)
  remover.run()


class ShopifyProductsBlacklistRemover():
  def __init__(self, store_code, product_service, broker_url, queue, qps):
    self.store_code = store_code
    self.product_service = product_service
    self.indice_name = 'shopify_products'
    self.broker_url = broker_url
    self.queue = queue
    self.qps = qps

  def run(self):
    records = dict()
    for keyword in BlacklistKeyword.select().dicts():
      k = keyword['keywords']
      records[k]= []

      query = {
        'query': {
          'query': {
            'multi_match': {
              'query': k,
              'type': 'phrase',
              'fields': ['source_product_id^10', 'handle^10', 'title^5', 'description']
            }
          }
        }
      }
      for prod_id, prod in self.product_service.load_products(self.indice_name, query):
        record = {
          's_product_id': prod['s_product_id'],
          'source': prod['source'],
          'source_product_id': prod['source_product_id'],
          'handle': prod['handle']
        }
        logger.debug(record)
        records[k].append(record)

    last_send_time = None
    connection = Connection(self.broker_url)
    for k, items in records.items():
      if not items:
        continue

      logger.debug(items)

      # handles = [item['handle'] for item in items]
      # gps = dict()
      # for gp in GmcProduct.select().where(GmcProduct.handle.in_(handles)).execute():
      #   gs_id = gp.gmc_store_id
      #   if gs_id in gps:
      #     gps[gs_id].append(gp.gmc_id)
      #   else:
      #     gps[gs_id] = [gp.gmc_id]

      # for gs_id, gmc_ids in gps.items():
      #   gmc_store = GmcStore.get_or_none(GmcStore.id == gs_id)
      #   if not gmc_store:
      #     continue

      #   gs = GoogleShopping(gmc_store.merchant_id, json.loads(gmc_store.service_account_data))
      #   gs.delete_products(gmc_ids)

      # for item in items:
      #   product_id = item['s_product_id']

      #   if last_send_time:
      #     wait_time = 1 / self.qps - (time.time() - last_send_time)
      #     if wait_time > 0:
      #       logger.debug("Waiting %.3fs to send next message", wait_time)
      #       time.sleep(wait_time)

      #   last_send_time = time.time()

      #   delete_shopify_product.apply_async(
      #     args=(self.store_code, product_id),
      #     queue=self.queue,
      #     connection=connection)
      #   logger.debug(
      #     'Added delete_shopify_product(%s, %s)',
      #     self.store_code, product_id)


if __name__ == '__main__':
  send_shopify_blacklisted_product_delete_task()